#!/bin/bash

[[ -w /etc/passwd ]] || {
  echo "${0##*/}: Only root can run this program, not ${LOGNAME:-$EUID}"
  exit 1
}

unset YES
while getopts y c ; do
  case $c in
    y) YES=1 ;;
  esac
done

EPAGES_PROJECT=${EPAGES_PROJECT:-epages6}
[ -e /etc/default/$EPAGES_PROJECT ] && . /etc/default/$EPAGES_PROJECT

[[ -n $EPAGES ]] && EPAGES_D=${EPAGES%/*} || EPAGES_D=/srv/epages
[[ -n $EPAGES_CONFIG ]] && DATABASE_D=$EPAGES_CONFIG/Database.d || DATABASE_D=$EPAGES_D/eproot/Shared/Config/Database.d

#
# ask_del
#

ask_del () {
  local YES=$1
  local WHAT=$2
  local PARTS=$(echo "$3" | sed 's,^,  ,')
  local ans

  echo -e "\nInstalled ePages $WHAT:" ; echo "$PARTS"
  [[ -n $YES ]] && return 0

  echo -en "\nDo you really want to delete those ePages $WHAT? (y/n): "
  read ans
  [[ $ans = y || $ans = Y ]] && return 0 || return 1
}

#
# stop epages
#

for i in $EPAGES_PROJECT mongod epagesj epages-solr epages-solr-slave ; do
  PERL5LIB= /sbin/chkconfig --del $i >/dev/null 2>&1
  [[ $i = $EPAGES_PROJECT && -x /etc/init.d/$i ]] && /etc/init.d/$i stop_cron >/dev/null
  [[ -x /etc/init.d/$i ]] && /etc/init.d/$i stop
done

#
# umount/unshare
#

if [[ -x /var/epages/epages-fs.sh ]] ; then
  /var/epages/epages-fs.sh -unmount 2>/dev/null
  /var/epages/epages-fs.sh -unshare 2>/dev/null
fi

#
# delete RPMs
#

unset INSTALLED
if type -p repoquery >/dev/null 2>&1 ; then
  INSTALLED=$(repoquery -a --qf "%{repoid} %{name}" | awk '/epages/{print $2}')
fi
if type -p yum >/dev/null 2>&1 ; then
  INSTALLED="$INSTALLED$'\n'"$(yum list installed | tr '\n' '"' | sed 's," ,,g' | tr '"' '\n' | awk '/epages/{sub(/\..*/,""); print $1}')
elif type -p zypper >/dev/null 2>&1 ; then
  INSTALLED=$(for i in $(cat /etc/zypp/repos.d/epages-*.repo \
              | awk -F\[ '/^[[:space:]]*\[epages.*[^0-9]\]/{print $2}') ; do
                zypper mr -e ${i%]}
                zypper search -i -r ${i%]}
                zypper mr -d ${i%]}
              done | awk '/^i \|/{print $3}')
fi
INSTALLED="$INSTALLED$'\n'"$(rpm -qa --qf '%{NAME}\n' | grep epages)

unset j
for i in $(echo "$INSTALLED" | sort -r | uniq) ; do
  rpm --quiet -q $i && j="$j $i"
done
INSTALLED=$j

if [[ -n $INSTALLED ]] ; then
  if ask_del "$YES" RPMs "$(echo $INSTALLED | tr ' ' '\n')" ; then
    [[ -x $EPAGES/bin/use-local-httpd.sh ]] && $EPAGES/bin/use-local-httpd.sh -u

    unset RELEASE_INSTALLED
    for i in $INSTALLED ; do
      [[ $i = epages-release ]] && { RELEASE_INSTALLED=1; continue; }
      echo rpm --nodeps -v -e $i
      rpm --nodeps -v -e $i
    done
  fi
  [[ -n $RELEASE_INSTALLED ]] && rpm --nodeps -v -e epages-release
fi

#
# delete users
#

mkdir -p "$EPAGES_D"/etc/
echo -e "user epagesj\ngroup epagesj" >> "$EPAGES_D"/etc/created-accounts
INSTALLED=$(sort -r "$EPAGES_D"/etc/created-accounts 2>/dev/null | uniq | while read type name ; do
  if [[ $type = user ]] ; then
    getent passwd $name >/dev/null 2>&1 && echo $type $name
  elif [[ $type = group ]] ; then
    getent group $name >/dev/null 2>&1 && echo $type $name
  fi
done)

if [[ -n $INSTALLED ]] ; then
  if ask_del "$YES" users/groups "$INSTALLED" ; then
    echo "$INSTALLED" | while read type name ; do
      if [[ $type = user ]] ; then
        echo userdel -r $name
        userdel -r $name
      elif getent group $name >/dev/null 2>&1 ; then
        echo groupdel $name
        groupdel $name
      fi
    done
  fi
fi

#
# delete databases
#

[ -w /etc/init.d/mysql ] && EPAGES_MYSQLD=mysql || EPAGES_MYSQLD=mysqld
if [[ -x /etc/init.d/$EPAGES_MYSQLD ]] && ls $DATABASE_D/*.conf >/dev/null 2>&1 ; then

  unset INSTALLED
  while read i ; do
    DATASOURCE=$(awk -F= "/^datasource=/{print \$2}" $i)
    [ "$DATASOURCE" = "${DATASOURCE#dbi:mysql:}" ] && continue
    DB=$(awk -F= "/^Database=/{print \$2}" $i)
    [ -z "$DB" ] && continue
    INSTALLED="$INSTALLED $DB"
  done < <(grep -l "^StoreType=" $DATABASE_D/*.conf)
  INSTALLED=$(echo $INSTALLED | tr ' ' '\n')

  if [[ -n $INSTALLED ]] ; then
    if ask_del "$YES" databases "$INSTALLED" ; then
      /etc/init.d/$EPAGES_MYSQLD stop >/dev/null 2>&1

      PATH=$PATH:/usr/bin:/bin
      mysqld_safe --skip-grant-tables --skip-networking >/dev/null 2>&1 &
      sleep 2

      for i in $INSTALLED ; do
        echo "drop database $i;"
        echo "drop database $i;" | mysql
      done

      echo /etc/init.d/$EPAGES_MYSQLD start
      pkill mysqld >/dev/null 2>&1
      sleep 2
      /etc/init.d/$EPAGES_MYSQLD start >/dev/null 2>&1
    fi
  fi
fi

#
# delete files
#

MONGOD=/etc/init.d/mongod
grep -q epages $MONGOD 2>/dev/null || unset MONGOD

unset INSTALLED
MORE="'/etc/default/$EPAGES_PROJECT*' '/etc/init.d/$EPAGES_PROJECT*' '/etc/sysconfig/$EPAGES_PROJECT*' '/etc/*/rc?.d/*$EPAGES_PROJECT*'"
[[ $EPAGES_PROJECT = ${EPAGES_PROJECT#epages} ]] || unset MORE
for i in $EPAGES_D \
    /etc/default/epages* \
    /etc/sysconfig/epages* \
    /etc/init.d/epages* \
    '/etc/*/rc?.d/*epages*' \
    $MORE \
    '/etc/yum.repos.d/*epages*' \
    '/etc/zypp/repos.d/*epages*' \
    /var/epages \
    /var/tmp/ini-config.sh \
    /var/tmp/epages-fs.sh $MONGOD
do
  eval ls -d1 $i >/dev/null 2>&1 && { [[ -z $INSTALLED ]] && INSTALLED="$i" || INSTALLED="$INSTALLED $i"; }
done
if ask_del "$YES" files "$(echo "$INSTALLED" | tr ' ' '\n')" ; then
  type -p yum >/dev/null 2>&1 && yum clean all || { zypper clean -a; zypper refresh; }
  echo rm -rf $INSTALLED
  eval rm -rf $INSTALLED
fi

exit 0
