#!/bin/env bash

BN=${0##*/}
PATH=/usr/sbin:$PATH

usage() {
  ( echo "Usage:"
    echo "  $BN {  -share | -mount FILESERVER | -unmount | -unshare }"
    echo "Options:"
    echo "  -share: share /srv/epages with other hosts"
    echo "  -mount: mount /srv/epages from FILESERVER"
    echo "  -unmount: unmount /srv/epages"
    echo "  -unshare: unshare /srv/epages" ) 1>&2
  exit 1
}

[ $# -eq 0 ] && usage

case "$1" in
  -share|-unshare|-unmount) TASK=${1#-} ;;
  -mount) TASK=${1#-} ; [ -z "$2" ] && usage ; FILESERVER=$2 ;;
  *) usage ;;
esac

case `uname -r` in
  2.6.16[.-]*) NFSSERVER=nfsserver ; INSTALL="zypper install nfs-utils" ;;
  2.6.18[.-]*) NFSSERVER=nfs ; INSTALL="yum install nfs-kernel-server" ;;
  2.6.27[.-]*) NFSSERVER=nfsserver ; INSTALL="zypper install nfs-utils" ;;
  2.6.32-*)    NFSSERVER=nfs ; INSTALL="yum install nfs-utils" ;;
  2.6.32.*)    NFSSERVER=nfsserver ; INSTALL="zypper install nfs-kernel-server" ;;
  3.0.*)    NFSSERVER=nfsserver ; INSTALL="zypper install nfs-kernel-server" ;;
esac

type -p exportfs 2>/dev/null || exit 0

if [[ $TASK = unmount ]] ; then
  echo /etc/init.d/epages6 stop
  /etc/init.d/epages6 stop >/dev/null
  mount | while read share on mount has type options ; do
   [[ $type = nfs && ${mount#/srv/epages} != $mount ]] || continue
   echo umount -f $mount
   umount -f $mount
  done
  grep -v " /srv/epages" /etc/fstab > /tmp/.~$$~
  cat /tmp/.~$$~ > /etc/fstab
  rm -f /tmp/.~$$~
  rm -f /etc/sysconfig/epages6
  exit 0
fi

if [[ $TASK = unshare ]] ; then
  grep -v /srv/epages /etc/exports > /tmp/.~$$~
  cat /tmp/.~$$~ > /etc/exports
  rm -f /tmp/.~$$~
  exportfs -r
  showmount -e
  exit 0
fi

if [[ $TASK = share ]] ; then
  rpm --quiet -q --whatprovides nfs-utils || {
    echo -e "No package provides nfs-utils\nFirst install NFS server with following command:\n  $INSTALL"
    exit 1
  }
  PATH=/usr/sbin:/sbin:$PATH
  PERL5LIB= chkconfig --list $NFSSERVER | grep -q '3:on' || {
    PERL5LIB= chkconfig --del $NFSSERVER >/dev/null
    [[ $NFSSERVER = nfs ]] && {
      RUN_SCRIPT=/etc/init.d/$NFSSERVER
      cp -p $RUN_SCRIPT $RUN_SCRIPT.tmp \
      && sed 's,\(chkconfig:\) -,\1 35,' $RUN_SCRIPT.tmp > $RUN_SCRIPT \
      && rm -f $RUN_SCRIPT.tmp
      chmod 755 $RUN_SCRIPT
    }
    PERL5LIB= chkconfig --add $NFSSERVER
  }
  pgrep nfsd >/dev/null 2>&1 || /etc/init.d/$NFSSERVER start
fi

for i in etc/sysconfig eproot/bin eproot/Perl eproot/Shared eproot/Cartridges ; do
  mkdir -p -m 755 /srv/epages/$i || exit 1
done

if [[ $TASK = share ]] ; then

  ls /etc/sysconfig/epages6 >/dev/null || exit 1

  if [ ! -f /srv/epages/etc/sysconfig/epages6 ] ; then
    rm -rf /srv/epages/etc/sysconfig/epages6
    cp -Lp /etc/sysconfig/epages6 /srv/epages/etc/sysconfig/epages6
    rm -f /etc/sysconfig/epages6
    ln -s /srv/epages/etc/sysconfig/epages6 /etc/sysconfig/epages6
  fi

  grep -q EPAGES_DISTRIBUTED_INSTALLATION=1 /etc/sysconfig/epages6 \
  || echo EPAGES_DISTRIBUTED_INSTALLATION=1 >> /etc/sysconfig/epages6

  if mount | grep " on /srv/epages/" || grep "^[^#]* /srv/epages/" /etc/fstab ; then
    echo "${0##*/}: Can't share mounted directories, exiting."
    exit 1
  fi

  grep -q '^/srv/epages ' /etc/exports || {
    FSID='fsid=0'
    grep -q $FSID /etc/exports && FSID= || FSID=',fsid=0'
    echo "/srv/epages *(async,rw,no_subtree_check,no_auth_nlm,no_root_squash$FSID)" >> /etc/exports
  }
  exportfs -r || exit 1
  showmount -e

else

  if exportfs 2>/dev/null | grep "^/srv/epages " || grep "^[[:space:]]*/srv/epages" /etc/exports 2>/dev/null ; then
    echo "${0##*/}: Can't mount own directories, exiting."
    exit 1
  fi

  host $FILESERVER || exit 1
  for i in etc/sysconfig eproot/bin eproot/Perl eproot/Shared eproot/Cartridges ; do
    grep -q "^[^#]* /srv/epages/$i " /etc/fstab || {
      echo "$FILESERVER:/srv/epages/$i /srv/epages/$i nfs async,exec,noatime,nodev,nolock,bg,hard,intr,rw,rsize=16384,wsize=16384 0 0" >> /etc/fstab
    }
    mount | grep -q " on /srv/epages/$i " || {
      echo mount /srv/epages/$i
      mount /srv/epages/$i || exit 1
    }
  done

  ls /srv/epages/etc/sysconfig/epages6 >/dev/null || exit 1

  grep -q EPAGES_DISTRIBUTED_INSTALLATION=1 /srv/epages/etc/sysconfig/epages6 \
  || echo EPAGES_DISTRIBUTED_INSTALLATION=1 >> /srv/epages/etc/sysconfig/epages6

  [[ $(find /etc/sysconfig/epages6 -printf "%l" 2>/dev/null) = /srv/epages/etc/sysconfig/epages6 ]] && exit 0

  rm -f /etc/sysconfig/epages6
  ln -s /srv/epages/etc/sysconfig/epages6 /etc/sysconfig/epages6 || exit 1

fi

exit 0
